#include <string>
#include <sstream>
#include <iostream>

using namespace std;

#define INFINITY        (1<<30)

#define EAST            0
#define WEST            1
#define NORTH           2
#define SOUTH           3

int min(int a, int b) { return (a < b) ? a : b; }

//The stop light delay function (also messy...)
int stoplight_delay(int stoplight, int d, int t)
{
    if(stoplight < 0 || t < stoplight)
        return t;
    
    int x = (t - stoplight) % 4;
    
    if((d & 2) == 0)
    {
        if(x >= 2)
            return t;
        
        return t + (2 - x);
    }
    else
    {
        if(x < 2)
            return t;
        return t + (4 - x);
    }
}

//Use Floyd-Warshall to get arrival times
void solve_grid(
    int arrival_times[8][8][4], 
    int v_speeds[8], 
    int h_speeds[8], 
    int stoplights[8][8])
{
    for(int k=0; k<256; k++)
    {
        for(int i=0; i<8; i++)
        for(int j=0; j<8; j++)
        for(int d=0; d<4; d++)
        {
            //Compute min arrival time for a[i][j][d]
            int best_t = arrival_times[i][j][d];
            
            for(int g=0; g<4; g++)
            {
                best_t = min(best_t, arrival_times[i][j][g]);
            }
            
            if(d == EAST && j > 0)
            {
                best_t = min(best_t, 
                    stoplight_delay(stoplights[i][j], d, h_speeds[i] + arrival_times[i][j-1][EAST]));
            }
            
            if(d == WEST && j < 7)
            {
                best_t = min(best_t, 
                    stoplight_delay(stoplights[i][j], d, h_speeds[i] + arrival_times[i][j+1][WEST]));
            }
            
            if(d == NORTH && i < 7)
            {
                best_t = min(best_t, 
                    stoplight_delay(stoplights[i][j], d, v_speeds[j] + arrival_times[i+1][j][NORTH]));
            }
            
            if(d == SOUTH && i > 0)
            {
                best_t = min(best_t, 
                    stoplight_delay(stoplights[i][j], d, v_speeds[j] + arrival_times[i-1][j][SOUTH]));
            }
            
            arrival_times[i][j][d] = best_t;
        }
    }
}


int main(int argc, char ** argv)
{
    int N;
    cin >> N;
    char buf[1024];
    cin.getline(buf, 1024);
    
    for(int casenum=1; casenum<=N; casenum++)
    {
        char input[4][1024];
        cin.getline(input[0], 1024);
        cin.getline(input[1], 1024);
        cin.getline(input[2], 1024);
        
        //Parse out data (what a mess...)
        stringstream ss;
        
        
        //Parse speed limits
        ss.clear();
        ss << input[0];
        
        int v_speeds[8], h_speeds[8];
        for(int i=0; i<8; i++)
        {
            v_speeds[i] = h_speeds[i] = 2;
        }
        char road;
        while(ss >> road)
        {
            if(road >= '1' && road <= '8')
                v_speeds[road - '1'] = 3;
            else if(road >= 'A' && road <= 'H')
                h_speeds[road - 'A'] = 3;
        }
        
        //Parse stoplights
        ss.clear();
        ss << input[1];
        
        int stoplights[8][8];
        int arrival_times[8][8][4];
        for(int i=0; i<8; i++)
        for(int j=0; j<8; j++)
        {
            stoplights[i][j] = -1;
            arrival_times[i][j][EAST] = 
            arrival_times[i][j][WEST] = 
            arrival_times[i][j][NORTH] = 
            arrival_times[i][j][SOUTH] = INFINITY;
        }
        char label[3];
        int delay;
        while(ss >> label >> delay)
        {
            stoplights[label[0] - 'A'][label[1] - '1'] = delay;
        }
        
        //Parse start location
        ss.clear();
        ss << input[2];
        
        char start_label[3], end_label[3];
        int start_time;
        ss >> start_label >> start_time >> end_label;

        int start_r = start_label[0] - 'A', 
            start_c = start_label[1] - '1';
        
        arrival_times[start_r][start_c][EAST] = 
        arrival_times[start_r][start_c][WEST] = 
        arrival_times[start_r][start_c][NORTH] = 
        arrival_times[start_r][start_c][SOUTH] = start_time;
        
        
        //Solve the damn thing
        solve_grid(arrival_times, v_speeds, h_speeds, stoplights);
        
        int end_r = end_label[0] - 'A', 
            end_c = end_label[1] - '1';
            
        int done_time = min(min(min(
            arrival_times[end_r][end_c][EAST],
            arrival_times[end_r][end_c][WEST]),
            arrival_times[end_r][end_c][NORTH]),
            arrival_times[end_r][end_c][SOUTH]) - start_time;
        
        cout << "Case " << casenum <<": " << done_time << " minutes" << endl << endl;
    }
    
    return 0;
}
