#include <iostream>
#include <sstream>
#include <cstdio>
#include <cstring>
#include <string>

using namespace std;


bool match_gene_type(string dna, string gene, size_t x, bool rev, bool transpose)
{
    if(dna.size() < x + gene.size())
        return false;
    
    for(size_t i=0; i<gene.size(); i++)
    {
        char c = gene[rev ? gene.size() - 1 - i : i];
        
        if(transpose)
        {
            if(c == 'T')      c = 'A';
            else if(c == 'C') c = 'G';
            else if(c == 'A') c = 'T';
            else if(c == 'G') c = 'C';
        }
        
        if(c != dna[i+x])
            return false;
    }
    
    return true;
}

bool match_gene(string dna, string gene, size_t x)
{
    //cout << "Checking gene " << gene << " at " << x << endl;
    
    return 
        match_gene_type(dna, gene, x, false, false) ||
        match_gene_type(dna, gene, x, false, true) ||
        match_gene_type(dna, gene, x, true, false) ||
        match_gene_type(dna, gene, x, true, true);
}

int main(int argc, char** argv)
{
    char gene_line[6000];
    int ngenes;
    string genes[30];
    string names[30];
    
    if(!(cin >> ngenes))
        return 0;
    
    memset(gene_line, 0, 6000);
    cin.getline(gene_line, 6000);
    
    for(int j=0; j<ngenes; j++)
    {
        memset(gene_line, 0, 6000);
        cin.getline(gene_line, 6000);
        int gene_start;
        
        for(int i=0; gene_line[i]; i++)
        {
            if(gene_line[i] == ':')
            {
                gene_line[i] = 0;
                gene_start = ++i;
                break;
            }
        }
        
        //Kill whitespace at EOL
        for(int i=gene_start; gene_line[i]; i++)
        {
            if(gene_line[i] >= 'A' && gene_line[i] <= 'Z')
                continue;
            gene_line[i] = 0;
        }
        
        
        names[j] = gene_line;
        genes[j] = &gene_line[gene_start];
    }
    
    
    for(int seqnum=1; ; seqnum++)
    {
        stringstream ss;
        string v;
        
        while(true)
        {
            cin >> v;
            ss << v;
            
            if(v[v.size() - 1] == '.')
                break;
        }
        
        string seq = ss.str();
        if(seq.size() == 1)
            return 0;
        
        bool has_match = false;
        printf("Sequence %d:", seqnum);
        
        for(int i=0; i<seq.size()-1; i++)
        {
            for(int j=0; j<ngenes; j++)
            {
                if(match_gene(seq, genes[j], i))
                {
                    cout << endl << "    " << names[j] << " at " << (i+1);
                    has_match = true;
                }
            }
        }
        
        if(!has_match)
            printf(" none");
        
        printf("\n\n");
    }
}
